local astar = {}
local nodes, nodeTmp

function astar.preset()
	nodes, nodeTmp = {}, {}

	local id = 1
	for y = 1, node.height do
		for x = 1, node.width do
			if node[y][x].is_walkable then
				nodes[#nodes + 1] =	{
									g		= 0,	--move cost, 10-horizontal/vertical, 14-diagonal, but it adds up later in calculations
									h		= 0,		--UZUPELNIC
									id		= id,
									node	= node[y][x],
									p		= -1,	--parent, -1 means no parent, otherwise it keeps parent's ID
									s		= 1,	--??????? to chyba status
									x		= x,
									y		= y
									}
				nodeTmp[x..","..y] = id
				id = id + 1
			end

		end
	end

	for i = 1, #nodes do
		for yy = -1, 1 do
			for xx = -1, 1 do
				local v = nodes[i]
				local key = v.x + xx..","..v.y + yy
				if nodeTmp[key] and (xx ~= 0 or yy ~= 0) then
					nodes[i][#nodes[i] + 1] = nodeTmp[key]	--adds neighbours' IDs to [nodes][i]
					nodes[i][#nodes[i] + 1] = xx * yy == 0 and 10 or 14	--adds [g] for these neighbours
				end
			end
		end
	end
end


function astar.generate(aEn, nEn, is_node)
	local nodeOp = {}

-- prepare (clear) all data from previous run
	for i = 1, #nodes do
		nodes[i].s = 1
		nodes[i].p = -1
		nodes[i].g = 0
		nodes[i].h = 0
	end
	
	local x, y = aEn.node.x, aEn.node.y
	local x2, y2 = 0, 0
	if is_node then 
		x2, y2 = nEn.x, nEn.y
	else
		x2, y2 = nEn.node.x, nEn.node.y
	end
	local abs = math.abs
	for i = 1, #nodes do
		nodes[i].h = (abs(x2 - x) + abs(y2 - y)) * 10
	end
	
	local id = nodeTmp[x..","..y]
	nodes[id].s = -1
	local active = nodes[id]

	repeat
-- each node from [nodes] which touches [active] add to [nodeOp] and then make [active] its parent and [g] = [g] parent + 10/14

		for i = 1, #active do
			if i % 2 == 1 then
				local key = active[i]
				if nodes[key].s == 1 then
					nodes[key].s = 0
					nodes[key].p = active.id
					nodes[key].g = active.g + active[i + 1]
					nodeOp[#nodeOp + 1] = nodes[key].id
				end
			end
		end

-- take the nodes that touch [active] and belong to [nodeOp] and have different parent (not [active])
-- if [g] of such nodes < [g] [active] + 10/14 (direction counted from [active]) then change its parent to [active] and update its [g]

		for i = 1, #active do
			if i % 2 == 1 then
				local key = active[i]
				if nodes[key].s == 0 and nodes[key].p ~= active.id then
					if active.g + active[i + 1] < nodes[key].g then
						nodes[key].p = active.id
						nodes[key].g = active.g + active[i + 1]
					end
				end
			end
		end

-- from [nodeOp] choose a nodes with smallest [g] + [h] and remove it from there. In [nodes] make its [status] = -1 and make it active

		local key, id = nil, nil
		for i = 1, #nodeOp do
			local v = nodeOp[i]
			if not id then
				key, id = i, v
			else
				if nodes[v].g + nodes[v].h < nodes[id].g + nodes[id].h then
					key, id = i, v
				end
			end
		end

		table.remove(nodeOp, key)
		nodes[id].s = -1
		active = nodes[id]
		x, y = active.x, active.y

	until x == x2 and y == y2
	aEn.path = {active.node}
	repeat
		local id = active.p
		active = nodes[id]
		aEn.path[#aEn.path + 1] = active.node
	until active.p == -1
	aEn.path[#aEn.path] = nil	--remove aEn position

end

return astar