local area, bsp = {}, {}
local random = math.random

function bsp.generate()
	area		=	{}
	nodeFloor	=	{}
	
	area[1] =	{
				ax		= 1,			--area x
				ay		= 1,			--area y
				aw		= node.width,	--area width
				ah		= node.height,	--area high
				lvl		= 0,			--bsp level of this area (area[1] has lvl 0 - no dividing) 
				rx		= -1,			--room x
				ry		= -1,			--room y
				rw		= -1,			--room width
				rh		= -1,			--room height
				state	= "open"		--open / divided / locked
				}
	
	local i = 1
	for y = 1, area[1].ah do
		node[y] = {}
		for x = 1, area[1].aw do
			node[y][x] =	{
							desc		= nil,		-- [str] description
							entity		= nil,		-- [tbl] pointer to entity
							graphic		= nil,		-- [num] number of quad
							info		= "void",	-- [str] void, floor/rip, wall/door
							item		= {},		-- [tbl] table of pointers to items (presentyly only 1 per node allowed)
							is_visited	= false,	-- [bln]
							is_visible	= false,	-- [bln]
							is_walkable	= false,	-- [bln]
							x			= x,		-- [num]
							y			= y			-- [num]
							}
			
			i = i + 1
		end
	end

	bsp.divide(1)

	for i = 1, #nodeFloor do
		local x = nodeFloor[i].x
		local y = nodeFloor[i].y
		local aNd = node[y][x]
		
		for dy = -1, 1 do
			for dx = -1, 1 do	--no need to check if y+dy or x+dx is within node borders because rooms cannot start/end at node borders
				nNd = node[y + dy][x + dx]
				if nNd.info == "void" then
					nNd.info = "wall"
					nNd.graphic = random(1, 8)
					if y + dy == 1 or y + dy == node.height or x + dx == 1 or x + dx == node.width then
						nNd.desc = "limit"
					end
				end
			end
		end
	end
end

function bsp.divide(a)	--a = area number, first call always with (1)
	local v = area[a]
	if v.lvl == node.bspMax then
		bsp.room(v)
		return
	else
		if v.state == "open" then
			local child = bsp.splitter(v)
			if #child > 0 then
				v.state = "divided"
				for i = 1, 2 do
					area[#area + 1] = child[i]
					bsp.divide(#area)
					if i == 2 then
						bsp.tunnel(child[1], child[2], a)
					end
				end
			else
				v.state = "locked"
				bsp.room(area[a])
			end
		end
	end
	return 
end



function bsp.splitter(v)
	local random = math.random
	local sMin, sMax, sPoint
	local ver, child = false, {}

	if v.aw >= v.ah then
		ver = true
		sMin = v.ax + node.roomMin + 1 + 2 * node.wShift
		sMax = v.aw - node.roomMin - 1 - 2 * node.wShift
	else
		sMin = v.ay + node.roomMin + 1 + 2 * node.wShift
		sMax = v.ah - node.roomMin - 1 - 2 * node.wShift
	end

	if sMin > sMax then 
		return child
	end
	
	sPoint = random(sMin, sMax)
	for i = 1, 2 do
		child[i] =	{
					lvl		= v.lvl + 1,
					rx		= -1,
					ry		= -1,
					rw		= -1,
					rh		= -1,
					state	= "open"
					}
	end
			
	if ver then
		child[1].ax, child[1].ay, child[1].aw, child[1].ah = v.ax, v.ay, sPoint, v.ah
		child[2].ax, child[2].ay, child[2].aw, child[2].ah = v.ax + sPoint - 1, v.ay, v.aw - sPoint + 1, v.ah
	else
		child[1].ax, child[1].ay, child[1].aw, child[1].ah = v.ax, v.ay, v.aw, sPoint
		child[2].ax, child[2].ay, child[2].aw, child[2].ah = v.ax, v.ay + sPoint - 1, v.aw, v.ah - sPoint + 1
	end

	return child
end



function bsp.room(v)
	if v.state ~= "divided" then
		local limitW = node.roomMax <= (v.aw - 2 - 2 * node.wShift) and node.roomMax or v.aw - 2 - 2 * node.wShift
		local limitH = node.roomMax <= (v.ah - 2 - 2 * node.wShift) and node.roomMax or v.ah - 2 - 2 * node.wShift
		
		v.rw = random(node.roomMin, limitW)
		v.rh = random(node.roomMin, limitH)
		v.rx = random(v.ax + 1 + node.wShift, v.ax + v.aw - 1 - v.rw - node.wShift)
		v.ry = random(v.ay + 1 + node.wShift, v.ay + v.ah - 1 - v.rh - node.wShift)
		
		for y = v.ry, v.ry + v.rh - 1 do
			for x = v.rx, v.rx + v.rw - 1 do
				local aNd = node[y][x]
				aNd.info = "floor"
				aNd.graphic = random(9, 16)
				aNd.is_walkable = true
				nodeFloor[#nodeFloor + 1] = node[y][x]
			end
		end
	end
end



function bsp.tunnel(A, B, p)	--[p] is hard to explain :)
	local xx, yy, x, y
	local dx, dy = 0, 0
	
	if random(0,1) == 0 then
		xx = random(A.rx, A.rx + A.rw - 1)
		yy = random(B.ry, B.ry + B.rh - 1)
	else
		xx = random(B.rx, B.rx + B.rw - 1)
		yy = random(A.ry, A.ry + A.rh - 1)
	end

	if node[yy][xx].info == "void" and not (yy >= A.ry and yy <= A.ry+A.rh-1) and not (xx >= B.rx and xx <= B.rx+B.rw-1) then
		if (B.rx > xx) and (A.ry < yy) then dx, dy = 1, -1  end
		if (B.rx > xx) and (A.ry > yy) then dx, dy = 1, 1 end
		if (B.rx < xx) and (A.ry < yy) then dx, dy = -1, -1 end
	end

	if node[yy][xx].info == "void" and not (yy >= B.ry and yy <= B.ry+B.rh-1) and not (xx >= A.rx and xx <= A.rx+A.rw-1) then
		if (A.rx < xx) and (B.ry > yy) then dx, dy = -1, 1  end
		if (A.rx < xx) and (B.ry < yy) then dx, dy = -1, -1 end
		if (A.rx > xx) and (B.ry > yy) then dx, dy = 1, 1 end
	end

	if dx * dy == 0 then
		if (A.rx + A.rw - 1) < B.rx then
			dx, dy = 1, 0	--horizontal tunnel
			xx = A.rx + A.rw
		else 
			dx, dy = 0, 1	--vertical tunnel
			yy = A.ry + A.rh
		end
	end
	
	if dx ~= 0 then		-- horizontal tunel in both cases (1 or 2 directional dig)
		x = xx
		if dy == 0 then
			repeat	--look for the "floor" to the left but ONLY when 1 directional dig
				x = x - 1
			until node[yy][x].info == "floor"
			x = x + 1	--just to go one node frw so it will start from void not floor
		end
		repeat
			local aNd = node[yy][x]
			if aNd.info == "void" then nodeFloor[#nodeFloor + 1] = node[yy][x] end
			aNd.info = "floor"
			aNd.graphic = random(9, 16)
			aNd.is_walkable = true
			x = x + dx
		until node[yy][x].is_walkable == true
	end
	
	if dy ~= 0 then
		y = yy
		if dx == 0 then
			repeat
				y = y - 1
			until node[y][xx].info == "floor"
			y = y + 1
		end
		repeat
			local aNd = node[y][xx]
			if aNd.info == "void" then nodeFloor[#nodeFloor + 1] = node[y][xx] end
			aNd.info = "floor"
			aNd.graphic = random(9, 16)
			aNd.is_walkable = true
			y = y + dy
		until node[y][xx].is_walkable == true
	end

	area[p].rx = A.rx <= B.rx and A.rx or B.rx
	area[p].ry = A.ry <= B.ry and A.ry or B.ry
	area[p].rw = A.rx + A.rw-1 >= B.rx + B.rw-1 and A.rx + A.rw - area[p].rx or B.rx + B.rw - area[p].rx
	area[p].rh = A.ry + A.rh-1 >= B.ry + B.rh-1 and A.ry + A.rh - area[p].ry or B.ry + B.rh - area[p].ry
end


return bsp