local item = {}

item	=	{
			data		= {},		-- [tbl] eg. {"nameJap", "nameEng"}
			desc		= nil,		-- [str]
			entity		= nil,		-- [tbl] entity object
			graphic		= 19,		-- [num]
			id			= nil,		-- [num]
			info		= nil,		-- [str]
			is_visible	= false,	-- [bln]
			is_walkable	= true,		-- [bln]
			node		= nil		-- [tbl] node object
			}

			
			
item.new = function(this, params)
	return setmetatable(params or {}, {__index = this})
end


item.putIntoMemory = function()
	local keys = {}
	for line in love.filesystem.lines(dict) do
		local first, second = item.mySplit(line, " - ")
		keys[#keys + 1] = {data = {first, second}}
	end
	return keys
end


item.putOnMap = function(this)
	local keys = {}
	for line in love.filesystem.lines(dict) do
		local first, second = item.mySplit(line, " - ")
		keys[#keys + 1] = {first, second}
	end
	if #keys < MON_NO then MON_NO = #keys end

	local nodeFloorLocal = {}
	for i = 1, #nodeFloor do nodeFloorLocal[i] = nodeFloor[i] end
	
	local random = math.random
	local rnd = random(MON_NO)	-- how many items to be put on ground

	for i = 1, rnd do
		local rn1 = random(#nodeFloorLocal)
		local rn2 = random(#keys)
		local aNd = nodeFloorLocal[rn1]
	
		this[i] = item:new(
							{
							data	= keys[rn2],
							id		= i,
							info	= "key",
							node	= aNd	-- but no entity
							}
							)
		
		aNd.item = {this[i]}
		
		table.remove(keys, rn2)
		table.remove(nodeFloorLocal, rn1)
	end
	return keys, nodeFloorLocal, rnd
end



item.putOnMonster = function(this, keys, nodeFloorLocal, itemsOnGround)
	local random = math.random

	for i = itemsOnGround + 1 , MON_NO do

		local rnd
		repeat rnd = random(2, #entity) until #entity[rnd].item == 0
		local rn1	= random(#keys)
		local aEn	= entity[rnd]
				
		this[i] = item:new(
							{
							data		= keys[rn1],
							entity		= aEn,	-- but no node
							id 			= i,
							info		= "key"
							}
							)

		aEn.autoPickCh = 0
		aEn.item = {this[i]}
		aEn.graphic = 17
		table.remove(keys, rn1)
	end
end



item.draw = function()
	local FS = node.size
	for i = 1, #entity do
		for j = 1, #entity[i].fov.item, 2 do
			local aIt = entity[i].fov.item[j]
			local dist = entity[i].fov.item[j + 1]
			if dist > 0.85 * entity[1].range then
				bright = 80
			elseif dist > 0.7 * entity[1].range then
				bright = 160
			else
				bright = 255
			end
			love.graphics.setColor(255, 255, 255, bright)
			local q = quads[aIt.graphic]
			local x = (aIt.node.x - 1) * FS
			local y = (aIt.node.y - 1) * FS
			love.graphics.drawq(tileset, q, x, y)
		end
		if not is_showAll then break end
	end
end


item.mySplit = function(s,sep)
	local a, b = string.find(s, sep)
	local first = string.sub(s, 1, a-1)
	local second = string.sub(s, b+3)
	return first, second
end



return item